@@ -8,17 +8,16 @@ use PIX::Object;
 use PIX::Accesslist;
 
 BEGIN {
-	use Exporter();
+	use Exporter;
 
 	our ($VERSION, @ISA, @EXPORT, @EXPORT_OK, %EXPORT_TAGS);
 
-	$VERSION = '1.03';
+	$VERSION = '1.10';
 
 	@ISA = qw(Exporter);
 	@EXPORT = qw();
 	@EXPORT_OK = qw();
 	%EXPORT_TAGS = ();
-
 }
 
 =pod
@@ -29,49 +28,78 @@ PIX::Walker - Process Cisco PIX configs and 'walk' access-lists
 
 =head1 SYNOPSIS
 
-PIX::Walker is an object that allows you to process PIX firewall configs and 'walk' an access-list for matches.
-PIX OS versions 6 and 7 are supported.
+PIX::Walker is an object that allows you to process PIX (and ASA) firewall
+configs and 'walk' an access-list for matches. PIX OS versions 6 and 7 are
+supported. Note, ACL's that use the 'interface' keyword will not match properly
+since there is no way for the Walker to match an IP to an interface, at least
+not yet.
 
-B<** This module is still in very early development **>
+'Loose' ACL matching performed. This means that you can specify as little as a
+single IP to match what line(s) that IP would match in the ACL on the firewall.
+Or you can provide every detail including source/dest IPs, ports, and protocol
+to match a specific line of an ACL. Loose matching allows you to see potential
+lines in a large ruleset that a single source or destination IP might match.
 
-'Loose' ACL matching performed. This means that you can specify as little as a source IP to match what line(s) that IP
-would match in the ACL on the firewall. Or you can provide every detail including source/dest IPs, ports, and protocol to 
-match a specific line of an ACL. Loose matching allows you to see potential lines in a large ruleset that a single source or
-destination IP might match.
-
-More than just the first line match can be returned. 
-If your search criteria can technically match multiple lines they will all be returned.
-This is useful for seeing extra matches in your ACL that might also match and can help you optimize your ACL.
+More than just the first line match can be returned. If your search criteria can
+technically match multiple lines they will all be returned. This is useful for
+seeing extra matches in your ACL that might also match and can help you optimize
+your ACL.
 
 =head1 EXAMPLE
 
   use PIX::Walker;
 
-  my $config = ' ... string of full firewall config ... ';
+  my $config = "... firewall config buffer or filename ...";
   my $fw = new PIX::Walker($config);
-  my $acl = $fw->acl('outside_access') || die("ACL does not exist");
+  my $acl = $fw->acl("outside_access") || die("ACL does not exist");
 
   my $matched = 0;
+  # search each line of the ACL for possible matches
   foreach my $line ($acl->lines) {
     if ($line->match(
-        source => '10.0.1.100', 
-        dest => '192.168.1.3', 
-        port => '80', 
-        protocol => 'tcp')) {
-      print "Matched ACL $acl->name ($acl->elements ACE)\n" if !$matched++;
+        source => "10.0.1.100", 
+        dest => "192.168.1.3", 
+        dport => "80",  	# dest port
+        proto => "tcp")) {
+      if (!$matched++) {
+        print "Matched ACL " . $acl->name .
+	  " (" . $acl->elements . " ACE)\n";
+      }
       print $line->print, "\n";
     }
   }
 
 =head1 METHODS
 
+=over
+
 =cut
 
+=item B<new($config, [$not_a_file])>
+
+=over
+
+Returns a new PIX::Walker object using the $config string passed in. The
+configuration is processed and broken out into various objects automatically.
+
+The $config string is either a full string buffer containing the configuration
+of a firewall or is used as a filename to read the configuration from, using
+various filename formats (tried with and without any extension on the filename)
+
+	* {$config}
+	* {$config}.conf
+	
+If $not_a_file is true then the $config string is never checked against the
+file system.
+
+=back
+
+=cut
 sub new {
 	my $proto = shift;
 	my $class = ref($proto) || $proto;
 	my $self = { debug => 0 };
-	my ($fw_config) = @_;
+	my ($fw_config, $not_a_file) = @_;
 	my $conf;
 	croak("Must provide firewall configuration") unless $fw_config;
 
@@ -84,24 +112,20 @@ sub new {
 	$file = (split(/\n/, $fw_config, 2))[0];
 	$host = (split(/\./, $file, 2))[0];
 
-	if (-f $file) {
-#		print "Reading from file\n";
-		open(F, "<$file") or die("Error opening file for reading: $!");
+	if (!$not_a_file and -f $file) {
+		open(F, "<$file") or croak("Error opening file for reading: $!");
 		$conf = join('', <F>);
 		close(F);
-	} elsif (-f "$file.conf") {
-#		print "Reading from conf file\n";
-		open(F, "<$file.conf") or die("Error opening file for reading: $!");
+	} elsif (!$not_a_file and -f "$file.conf") {
+		open(F, "<$file.conf") or croak("Error opening file for reading: $!");
 		$conf = join('', <F>);
 		close(F);
-	} elsif (-f "$host") {
-		print "Reading from host file\n";
-#		open(F, "<$host") or die("Error opening file for reading: $!");
+	} elsif (!$not_a_file and -f $host) {
+		open(F, "<$host") or croak("Error opening file for reading: $!");
 		$conf = join('', <F>);
 		close(F);
-	} elsif (-f "$host.conf") {
-#		print "Reading from host conf file\n";
-		open(F, "<$host.conf") or die("Error opening file for reading: $!");
+	} elsif (!$not_a_file and -f "$host.conf") {
+		open(F, "<$host.conf") or croak("Error opening file for reading: $!");
 		$conf = join('', <F>);
 		close(F);
 	} else {
@@ -132,6 +156,7 @@ sub _init {
 		'citrix-ica'		=> '1494',
 
 		# cisco PIX defined
+		# (there may be more now; I have not updated this in awhile)
 		'aol'			=> '5190',
 		'bgp'			=> '179',
 		'biff'			=> '512',
@@ -139,6 +164,7 @@ sub _init {
 		'bootps'		=> '67',
 		'chargen'		=> '19',
 		'cmd'			=> '514',
+		'rsh'			=> '514',
 		'daytime'		=> '13',
 		'discard'		=> '9',
 		'domain'		=> '53',
@@ -172,6 +198,7 @@ sub _init {
 		'pop2'			=> '109',
 		'pop3'			=> '110',
 		'pptp'			=> '1723',
+		'radius-acct'		=> '1813',
 		'rip'			=> '520',
 		'rtsp'			=> '554',
 		'sip'			=> '5060',
@@ -193,12 +220,12 @@ sub _init {
 		'xdmcp'			=> '177',
 	};
 
-	# look for services files (nmap is better) and build a translation table.
-	# this reads ALL the files listed and merges the results into a single hash lookup table.
-	# the first name-to-port lookup found is used and is not overwritten
-#	my @files = qw( ./port-numbers /usr/local/share/nmap/nmap-services /usr/share/nmap/nmap-services /etc/services );
+	# Look for services files (nmap is better) and build a translation
+	# table. This reads ALL the files listed and merges the results into a
+	# single hash lookup table. the first name-to-port lookup found is used
+	# and is not overwritten. This obviously only works on Linux.
 	my @files = qw( /usr/local/share/nmap/nmap-services /usr/share/nmap/nmap-services /etc/services );
-	while ( defined(my $file = shift @files)) {
+	while (defined(my $file = shift @files)) {
 		next unless -f $file;
 		open(F, "<$file") or next;
 		while (defined(my $line = <F>)) {
@@ -210,7 +237,6 @@ sub _init {
 			$self->{ports}{$name} = $port unless exists $self->{ports}{$name};
 		}
 		close(F);
-#		last;	# uncomment this to only read the FIRST file found
 	}
 }
 
@@ -229,7 +255,7 @@ sub _process {
 				$line = $self->_nextline;
 			}
 			$self->_rewind($line);		# rewind 1 line so we don't skip past it on the next iteration
-			$self->{objects}{$name} = new PIX::Object($type, $name, $conf, $self);
+			$self->add_obj($type, $name, $conf);
 		} elsif ($line =~ /^access-list (\S+)/) {
 			my $name = $1;
 			next if $name eq 'compiled';
@@ -240,9 +266,9 @@ sub _process {
 				$line = $self->_nextline;
 			}
 			$self->_rewind($line);
-			$self->{acls}{$name} = new PIX::Accesslist($name, $conf, $self);			
+			$self->add_acl($name, $conf);
 
-		} elsif ($line =~ /^name (\S+) (.+)/) {
+		} elsif ($line =~ /^name (\S+) (\S+)/) { # ignore descriptions
 			$self->{alias}{$2} = $1;
 		}
 	}
@@ -253,7 +279,7 @@ sub _process {
 
 =over
 
-Returns an B<PIX::Accesslist> object for the ACL named by $name.
+Returns an PIX::Accesslist object for the ACL named by $name.
 
 =back
 
@@ -268,16 +294,47 @@ sub acl {
 
 =over
 
-Returns an array of PIX::Accesslist objects for each access-list 
+Returns an array of access-list strings for each access-list 
 found in the firewall configuration. Returns undef if there is no
-matching ACL.
+matching ACL. Use walker->acl('acl_name') to retrieve the actual
+PIX::Accesslist object.
 
 =back
 
 =cut
 sub acls { keys %{$_[0]->{acls}} }
 
-=item B<alias($string)>
+=item B<add_acl($name, [\@conf])>
+
+=over
+
+Add's an access-list object to the PIX::Walker object. $conf is an arrayref
+of the configuration lines that make up the access-list and can be empty.
+
+=back
+
+=cut
+sub add_acl {
+	my ($self, $name, $conf) = @_;
+	return $self->{acls}{$name} = new PIX::Accesslist($name, $conf || [], $self);
+}
+
+=item B<add_obj($name, $type, [\@conf])>
+
+=over
+
+Add's an object-group object to the PIX::Walker object. $conf is an arrayref
+of the configuration lines that make up the object-group and can be empty.
+
+=back
+
+=cut
+sub add_obj {
+	my ($self, $type, $name, $conf) = @_;
+	return $self->{objects}{$name} = new PIX::Object($type, $name, $conf || [], $self);
+}
+
+=item B<alias($alias)>
 
 =over
 
@@ -289,8 +346,8 @@ string is returned unchanged.
 =cut
 sub alias {
 	my $self = shift;
-	my $string = shift;
-	return exists $self->{alias}{$string} ? $self->{alias}{$string} : $string;
+	my $alias = shift;
+	return exists $self->{alias}{$alias} ? $self->{alias}{$alias} : $alias;
 }
 
 =item B<findip($ip, [$trace])>
@@ -304,6 +361,7 @@ array reference is returned containing all matches.
 * I<$ip> is an IP address to look for.
 
 * I<$trace> is an optional reference to a trace buffer. 
+
 If an IP is found in a nested group the trace will allow you to find out where 
 it was nested. See L<tracedump()> for more information.
 
@@ -312,7 +370,7 @@ it was nested. See L<tracedump()> for more information.
 =cut
 sub findip {
 	my ($self, $ip, $trace) = @_;
-	my $found = [];
+	my $found = [];	
 
 	foreach my $obj (keys %{$self->{objects}}) {
 		my $grp = $self->{objects}{$obj};
@@ -343,6 +401,7 @@ array reference is returned containing all matches.
 * I<$port> is the PORT to look for.
 
 * I<$trace> is an optional reference to a trace buffer. 
+
 If a PORT is found in a nested group the trace will allow you to find out where 
 it was nested. See L<tracedump()> for more information.
 
@@ -386,7 +445,43 @@ sub obj {
 	return exists $self->{objects}{$name} ? $self->{objects}{$name} : undef;
 }
 
+=item B<objs([$type])>
 
+=over
+
+Returns an array of object-group strings for each object-group found in the
+firewall configuration. If $type is specified only groups matching that type
+are returned.
+
+Returns undef if there are no object-groups. Use walker->obj('obj_name') to
+retreive the actual PIX::Object object.
+
+=back
+
+=cut
+sub objs {
+	my ($self, $type) = @_;
+	$type = lc $type;
+	if ($type) {
+		return grep { $self->{objects}{$_}->type eq $type } keys %{$self->{objects}};
+	} else {
+		return keys %{$self->{objects}};
+	}
+}
+
+=item B<portnum($port)>
+
+=over
+
+Returns the port NUMBER of the port name given. This function will DIE() if the
+port name is not known. This is harsh because the routines that use this
+function will not work if a single port lookup fails (not being able to lookup
+a port number can cause some of your acl searching to fail). This function is
+meant to be used internally only.
+
+=back
+
+=cut
 sub portnum { 
 	my ($self, $port) = @_;
 	return $port if $port =~ /^\d+$/;
@@ -448,11 +543,9 @@ sub total_object_groups { return scalar keys %{$_[0]->{objects}} }
 
 1;
 
-__DATA__
-
 =head1 AUTHOR
 
-Jason Morriss, C<< <lifo at liche.net> >>
+Jason Morriss <lifo 101 at - gmail dot com>
 
 =head1 BUGS
 
@@ -464,15 +557,25 @@ your bug as I make changes.
 
 =head1 SUPPORT
 
-This POD document is the only support you will receive on this module.
+    perldoc PIX::Walker
+
+    perldoc PIX::Accesslist
+    perldoc PIX::Accesslist::Line
+
+    perldoc PIX::Object
+    perldoc PIX::Object::network
+    perldoc PIX::Object::service
+    perldoc PIX::Object::protocol
+    perldoc PIX::Object::icmp_type
 
 =head1 ACKNOWLEDGEMENTS
 
-B<Peter Vargo> - For pushing me to make this module and for supplying me with endless ideas.
+B<Peter Vargo> - For pushing me to make this module and for supplying me with
+endless ideas.
 
 =head1 COPYRIGHT & LICENSE
 
-Copyright 2006 Jason Morriss, all rights reserved.
+Copyright 2006-2008 Jason Morriss, all rights reserved.
 
 This program is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
